@@ -1,7 +1,5 @@
 import torch
 from torch.utils.data import DataLoader, TensorDataset
-import numpy as np
-import pandas as pd
 import torchsummary
 from collections import OrderedDict
 from torch.utils.data import random_split
@@ -12,8 +10,9 @@ from .metrics import Metric
 from .metrics import create_metric_by_name
 from .losses import create_loss_by_name
 from .optimizers import create_optimizer_by_name
+from .utils import to_tensor
 
-__version__ = '0.4.6'
+__version__ = '0.6.3'
 
 class Model(torch.nn.Module):
     """
@@ -36,20 +35,6 @@ class Model(torch.nn.Module):
     def forward(self, x):
         return self.model.forward(x)
 
-    @staticmethod
-    def to_tensor(*args):
-        rt = []
-        for arg in args:
-            if isinstance(arg, pd.DataFrame):
-                arg = torch.from_numpy(arg.values)
-            elif isinstance(arg, np.ndarray):
-                arg = torch.from_numpy(arg)
-            elif not isinstance(arg, torch.Tensor):
-                raise TypeError('Only DataFrame, ndarray and torch.Tensor are supported.')
-            rt.append(arg)
-                
-        return rt[0] if len(rt) == 1 else tuple(rt)
-
     def count_params(self) -> int:
         """Count the total number of scalars composing the weights."""
         return sum([p.numel() for p in self.parameters()])
@@ -66,10 +51,13 @@ class Model(torch.nn.Module):
         self.built = True
         return self
 
-    def summary(self, input_shape, depth=3):
-        """Print a string summary of the network."""
+    def _check_keras_layer(self):
         if self.has_keras_layer and not self.built:
             raise AssertionError("You should call `model.build()` first because the model contains `KerasLayer`.")
+
+    def summary(self, input_shape, depth=3):
+        """Print a string summary of the network."""
+        self._check_keras_layer()
         torchsummary.summary(self.model, input_shape, depth=depth, verbose=1)
 
     def compile(self, optimizer, loss, metrics=None, device=None):
@@ -78,18 +66,19 @@ class Model(torch.nn.Module):
 
         Args:
 
-        * :attr:`optimizer`: String (name of optimizer) or optimizer instance.
+        * `optimizer`: String (name of optimizer) or optimizer instance.
 
-        * :attr:`loss`: String (name of objective function), objective function or loss instance.
+        * `loss`: String (name of objective function), objective function or loss instance.
 
-        * :attr:`metrics`: List of metrics to be evaluated by the model during training. You can also use dict to specify the 
+        * `metrics`: List of metrics to be evaluated by the model during training. You can also use dict to specify the 
         abbreviation of each metric.
 
-        * :attr:`device`: Device the model will run on, if `None` it will use 'cuda' when `torch.cuda.is_available()` otherwise 'cpu'.
+        * `device`: Device of the model and its trainer, if `None` 'cuda' will be used when `torch.cuda.is_available()` otherwise 'cpu'.
         """
-        if self.has_keras_layer and not self.built:
-            raise AssertionError("You should call `model.build()` first because the model contains `KerasLayer`.")
-        
+        self._check_keras_layer()
+        if device == None:
+            device = 'cuda' if torch.cuda.is_available() else 'cpu'
+            
         if isinstance(loss, str):
             loss = create_loss_by_name(loss)
         if isinstance(optimizer, str):
@@ -104,6 +93,7 @@ class Model(torch.nn.Module):
                     tmp_m = create_metric_by_name(tmp_m)
                 if isinstance(tmp_m, Metric):
                     m[tmp_m.get_abbr()] = tmp_m
+                    tmp_m.set_device(device)
                 elif hasattr(tmp_m, '__call__'):
                     m[tmp_m.__name__] = tmp_m
                 else:
@@ -111,8 +101,6 @@ class Model(torch.nn.Module):
         else:
             raise TypeError('Argument `metrics` should be either a dict or list.')
 
-        if device == None:
-            device = 'cuda' if torch.cuda.is_available() else 'cpu'
         self.to(device=device)
         self.trainer = Trainer(model=self, optimizer=optimizer, loss=loss, metrics=m, device=device)
         self.compiled = True
@@ -123,20 +111,28 @@ class Model(torch.nn.Module):
                 validation_data=None,
                 callbacks=[],
                 verbose=1,
-                precise_mode=False,
+                precise_train_metrics=False,
+                shuffle=True,
+                sample_weight=None
                 ):
         """Train the model for a fixed number of epochs (iterations on a dataset)."""
 
         assert self.compiled
-        x, y = self.to_tensor(x, y)
+        x, y = to_tensor(x, y)
 
         assert not (validation_data != None and validation_split != None)
         has_val = validation_data != None or validation_split != None
 
-        train_set = TensorDataset(x, y)
-    
+        if type(sample_weight) != type(None):
+            if isinstance(sample_weight, list):
+                sample_weight = torch.tensor(sample_weight)
+            sample_weight = to_tensor(sample_weight).float()
+            train_set = TensorDataset(x, y, sample_weight)
+        else:
+            train_set = TensorDataset(x, y)
+
         if validation_data != None:
-            x_val, y_val = self.to_tensor(validation_data[0], validation_data[1])
+            x_val, y_val = to_tensor(validation_data[0], validation_data[1])
             val_set = TensorDataset(x_val, y_val)
 
         if validation_split != None:
@@ -144,12 +140,12 @@ class Model(torch.nn.Module):
             train_length = len(train_set) - val_length
             train_set, val_set = random_split(train_set, [train_length, val_length], generator=torch.Generator().manual_seed(val_split_seed))
 
-        train_loader = DataLoader(train_set, batch_size=batch_size, shuffle=True)
+        train_loader = DataLoader(train_set, batch_size=batch_size, shuffle=shuffle)
         val_loader = DataLoader(val_set, batch_size=batch_size, shuffle=False) if has_val else None
 
         # Training
         self.trainer.register_callbacks(callbacks)
-        history = self.trainer.run(train_loader, val_loader, max_epochs=epochs, verbose=verbose, precise_mode=precise_mode)
+        history = self.trainer.run(train_loader, val_loader, max_epochs=epochs, verbose=verbose, precise_train_metrics=precise_train_metrics)
 
         return history
 
@@ -157,15 +153,14 @@ class Model(torch.nn.Module):
     def evaluate(self, x, y, batch_size=32):
         """Return the loss value & metrics values for the model in test mode.\n\n    Computation is done in batches."""
         assert self.compiled
-        x, y = self.to_tensor(x, y)
+        x, y = to_tensor(x, y)
         val_loader = DataLoader(TensorDataset(x, y), batch_size=batch_size, shuffle=False)
         return self.trainer.evaluate(val_loader)
 
     @torch.no_grad()
-    def predict(self, inputs, batch_size=32, device=None, output_numpy=True):
+    def predict(self, inputs, batch_size=32, device=None, output_numpy=True, activation=None):
         """Generate output predictions for the input samples.\n\n    Computation is done in batches."""
-        if self.has_keras_layer and not self.built:
-            raise AssertionError("You should call `model.build()` first because the model contains `KerasLayer`.")
+        self._check_keras_layer()
 
         if device == None:
             if self.compiled:
@@ -173,7 +168,7 @@ class Model(torch.nn.Module):
             else:
                 device = 'cuda' if torch.cuda.is_available() else 'cpu'
 
-        inputs = self.to_tensor(inputs)
+        inputs = to_tensor(inputs)
         self.eval().to(device=device)
         outputs = []
 
@@ -182,6 +177,10 @@ class Model(torch.nn.Module):
             outputs.append(self.forward(x_batch[0].to(device=device)))
 
         outputs = torch.cat(outputs, dim=0)
+
+        if activation != None:
+            outputs = activation(outputs)
+
         if output_numpy:
             return outputs.cpu().numpy()
         else:
@@ -192,5 +191,5 @@ class Model(torch.nn.Module):
         torch.save(self.state_dict(), filepath)
 
     def load_weights(self, filepath):
-        """Equal to `model.load_state_dict(filepath)`."""
-        self.load_state_dict(filepath)
\ No newline at end of file
+        """Equal to `model.load_state_dict(torch.load(filepath))`."""
+        self.load_state_dict(torch.load(filepath))
\ No newline at end of file
